require 'puppetx/puppetlabs/pe_console/config'
require 'puppetx/puppetlabs/pe_console/ssl/validator/certname_validator'
require 'puppet/network/http/connection'
require 'puppet/network/http/compression'

module Puppetx::Puppetlabs::PEConsole::ConsoleHTTP
  include Puppet::Network::HTTP::Compression.module

  def do_request
    connection = new_connection

    # Make the request, but hold off processing it until we verify the console identity
    response = yield(connection, get_default_headers)

    # Verify the dashboard certificate has been signed by a valid CA
    unless validator.valid_peer?
      raise Puppet::Error, "Could not validate console certificate."
    end

    # Verify the certificate's common name
    unless validator.certificate_name_match?
      raise Puppet::Error, "Console certificate name did not match expected CN: #{certificate_name}"
    end

    if response.is_a? Net::HTTPSuccess
      uncompress_body(response)
    else
      message = "Error #{response.code} while communicating with #{server} on port #{port}:\n" +
                "#{(response.body||'').empty? ? response.message : uncompress_body(response)}"
      raise Puppet::Error, message
    end
  end

  private

  ## Connection helpers ##

  # Creates a new connection using our certname validator
  def new_connection
    Puppet::Network::HttpPool.http_ssl_instance(server, port, validator)
  end

  # Provide appropriate headers.
  def get_default_headers
    add_accept_encoding({"Accept" => 'text/yaml'})
  end


  ## Configuration helpers ##

  def ssl_configuration
    @ssl_configuration ||= Puppet::SSL::Configuration.new(
      Puppet[:localcacert],
      {
        :ca_chain_file => Puppet[:ssl_client_ca_chain],
        :ca_auth_file  => Puppet[:ssl_client_ca_auth]
      })
  end

  def validator
    @validator ||= Puppetx::Puppetlabs::PEConsole::SSL::Validator::CertnameValidator.new(
      ssl_configuration,
      Puppet::SSL::Host.localhost,
      certificate_name)
  end

  def load_config
    @console_config = Puppetx::Puppetlabs::PEConsole::Config.load
  end

  def config
    @console_config
  end

  def server
    @console_config.server
  end

  def certificate_name
    @console_config.certificate_name
  end

  def port
    @console_config.port
  end

end
